@@ -40,13 +40,14 @@ from ..utils import (
 
 from ..debugging import log
 
+import time
 import random
 import re
+import hashlib
 from requests.exceptions import RequestException
 from json.decoder import JSONDecodeError
 from urllib import parse
 from datetime import datetime
-from base64 import b64decode
 
 
 class YouTubeChatDownloader(BaseChatDownloader):
@@ -260,20 +261,50 @@ class YouTubeChatDownloader(BaseChatDownloader):
             'expected_result': {
                 'error': VideoUnplayable,
             }
+        },
+
+        # Potential parsing errors
+        {
+            'name': "Parsing error with '};' inside yt initial data (1)",
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=CHqg6qOn4no',
+            },
+            'expected_result': {
+                'error': NoChatReplay,
+            }
+        },
+        {
+            'name': "Parsing error with '};' inside yt initial data (2)",
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=gVfgbahppCY',
+            },
+            'expected_result': {
+                'error': NoChatReplay,
+            }
+        },
+        {
+            'name': 'Title with JS-like syntax "};"',
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=lsguqyKfVQg',
+            },
+            'expected_result': {
+                'error': NoChatReplay,
+            }
         }
     ]
 
-    _YT_INITIAL_DATA_RE = r'(?:window\s*\[\s*["\']ytInitialData["\']\s*\]|ytInitialData)\s*=\s*({.+?})\s*;'
-    _YT_INITIAL_PLAYER_RESPONSE_RE = r'ytInitialPlayerResponse\s*=\s*({.+?})\s*;'
+    _YT_INITIAL_BOUNDARY_RE = r'\s*(?:var\s+meta|</script|\n)'
+    _YT_INITIAL_DATA_RE = r'(?:window\s*\[\s*["\']ytInitialData["\']\s*\]|ytInitialData)\s*=\s*({.+?})\s*;' + \
+        _YT_INITIAL_BOUNDARY_RE
+    _YT_INITIAL_PLAYER_RESPONSE_RE = r'ytInitialPlayerResponse\s*=\s*({.+?})\s*;' + \
+        _YT_INITIAL_BOUNDARY_RE
     _YT_CFG_RE = r'ytcfg\.set\s*\(\s*({.+?})\s*\)\s*;'
 
     _YT_HOME = 'https://www.youtube.com'
     _YT_VIDEO_TEMPLATE = _YT_HOME + '/watch?v={}'
 
     _YOUTUBE_INIT_API_TEMPLATE = _YT_HOME + '/{}?continuation={}'
-    _YOUTUBE_CHAT_API_TEMPLATE = _YT_HOME + \
-        b64decode(
-            'L3lvdXR1YmVpL3YxL2xpdmVfY2hhdC9nZXRfe30/a2V5PUFJemFTeUFPX0ZKMlNscVU4UTRTVEVITEdDaWx3X1k5XzExcWNXOA==').decode()
+    _YOUTUBE_CHAT_API_TEMPLATE = _YT_HOME + '/youtubei/v1/live_chat/get_{}?key={}'
 
     _MESSAGE_GROUPS = {
         'messages': [
@@ -1017,23 +1048,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
             if playlist_video:
                 yield self._parse_video(playlist_video)
 
-            # "continuationItemRenderer":{
-            #     "trigger":"CONTINUATION_TRIGGER_ON_ITEM_SHOWN",
-            #     "continuationEndpoint":{
-            #         "clickTrackingParams":"CCgQ7zsYACITCNT0zMKim-4CFU-V1QodgR4KyA==",
-            #         "commandMetadata":{
-            #             "webCommandMetadata":{
-            #             "sendPost":true,
-            #             "apiUrl":"/youtubei/v1/browse"
-            #             }
-            #         },
-            #         "continuationCommand":{
-            #             "token":"4qmFsgJhEiRWTFBMRXJ1a1gxVzFPWWpGeDJwRzh6aldpTXVQTUcwRi1MYkkaFENBRjZCbEJVT2tOSFVRJTNEJTNEmgIiUExFcnVrWDFXMU9ZakZ4MnBHOHpqV2lNdVBNRzBGLUxiSQ%3D%3D",
-            #             "request":"CONTINUATION_REQUEST_TYPE_BROWSE"
-            #         }
-            #     }
-            # }
-
     _CONSENT_ID_REGEX = r'PENDING\+(\d+)'
     # https://github.com/ytdl-org/youtube-dl/blob/a8035827177d6b59aca03bd717acb6a9bdd75ada/youtube_dl/extractor/youtube.py#L251
 
@@ -1058,6 +1072,25 @@ class YouTubeChatDownloader(BaseChatDownloader):
         self.set_cookie_value('.youtube.com', 'CONSENT',
                               'YES+cb.20210328-17-p0.en+FX+{}'.format(consent_id))
 
+    def _generate_sapisidhash_header(self):
+        sapis_id = self.get_cookie_value('SAPISID')
+        sapisid_cookie = self.get_cookie_value('__Secure-3PAPISID') or sapis_id
+
+        if sapisid_cookie is None:
+            return
+
+        time_now = round(time.time())
+
+        # SAPISID cookie is required if not already present
+        if not sapis_id:
+            self.set_cookie_value(
+                '.youtube.com', 'SAPISID', sapisid_cookie, secure=True, expire_time=time_now + 3600)
+
+        # SAPISIDHASH algorithm from https://stackoverflow.com/a/32065323
+        sapisidhash = hashlib.sha1(
+            f'{time_now} {sapisid_cookie} {self._YT_HOME}'.encode('utf-8')).hexdigest()
+        return f'SAPISIDHASH {time_now}_{sapisidhash}'
+
     def _get_initial_info(self, url):
         html = self._session_get(url).text
         yt = re.search(self._YT_INITIAL_DATA_RE, html)
@@ -1074,6 +1107,11 @@ class YouTubeChatDownloader(BaseChatDownloader):
             raise ParsingError(
                 'Unable to parse initial video data. {}'.format(html))
 
+        details = {}
+
+        cfg = re.search(self._YT_CFG_RE, html)
+        details['ytcfg'] = try_parse_json(cfg.group(1)) if cfg else {}
+
         player_response = re.search(self._YT_INITIAL_PLAYER_RESPONSE_RE, html)
         player_response_info = try_parse_json(
             player_response.group(1)) if player_response else None
@@ -1086,9 +1124,8 @@ class YouTubeChatDownloader(BaseChatDownloader):
         formats = streaming_data.get(
             'adaptiveFormats') or streaming_data.get('formats')
 
-        details = {
-            'start_time': float_or_none(multi_get(formats, 0, 'lastModified')),
-        }
+        details['start_time'] = float_or_none(
+            multi_get(formats, 0, 'lastModified'))
 
         # Try to get continuation info
         contents = yt_initial_data.get('contents') or {}
@@ -1197,19 +1234,12 @@ class YouTubeChatDownloader(BaseChatDownloader):
         if not is_live:
             api_type += '_replay'
 
-        init_page = self._YOUTUBE_INIT_API_TEMPLATE.format(
-            api_type, continuation)
-        # must run to get first few messages, otherwise might miss some
-        html, yt_info = self._get_initial_info(init_page)
+        yt_cfg_data = initial_info.get('ytcfg')
 
-        cfg = re.search(self._YT_CFG_RE, html)
-        yt_cfg_data = try_parse_json(cfg.group(1)) if cfg else {}
+        api_key = yt_cfg_data.get('INNERTUBE_API_KEY')
 
-        continuation_params = {
-            'context': yt_cfg_data.get('INNERTUBE_CONTEXT') or {}
-        }
-
-        continuation_url = self._YOUTUBE_CHAT_API_TEMPLATE.format(api_type)
+        continuation_url = self._YOUTUBE_CHAT_API_TEMPLATE.format(
+            api_type, api_key)
         offset_milliseconds = (
             start_time * 1000) if isinstance(start_time, (float, int)) else None
 
@@ -1230,29 +1260,69 @@ class YouTubeChatDownloader(BaseChatDownloader):
         self.check_for_invalid_types(
             messages_types_to_add, self._MESSAGE_TYPES)
 
+        click_tracking_params = None
+
+        innertube_context = yt_cfg_data.get('INNERTUBE_CONTEXT') or {}
+
+        self.update_session_headers({
+            'x-goog-visitor-id': multi_get(innertube_context, 'client', 'visitorData'),
+            'x-youtube-client-name': str(yt_cfg_data.get('INNERTUBE_CONTEXT_CLIENT_NAME')),
+            'x-youtube-client-version': str(yt_cfg_data.get('INNERTUBE_CLIENT_VERSION')),
+            'origin': self._YT_HOME,
+            'content-type': 'application/json'
+        })
+
         message_count = 0
         first_time = True
         while True:
+            continuation_params = {
+                'context': innertube_context,
+                'continuation': continuation
+            }
+
+            # Update authentication header, if necessary
+            auth = self._generate_sapisidhash_header()
+            if auth:
+                self.update_session_headers({
+                    'authorization': auth,
+                    'x-origin': self._YT_HOME
+                })
+
             info = None
             for attempt_number in attempts(max_attempts):
 
                 try:
+                    if first_time:
+                        # must run to get first few messages, otherwise might miss some
+                        init_page = self._YOUTUBE_INIT_API_TEMPLATE.format(
+                            api_type, continuation)
+                        html, yt_info = self._get_initial_info(init_page)
 
-                    if not first_time:
-
-                        continuation_params['continuation'] = continuation
-
+                    else:
                         if not is_live and offset_milliseconds is not None:
                             continuation_params['currentPlayerState'] = {
                                 'playerOffsetMs': offset_milliseconds}
-                        log('debug', 'Continuation: {}'.format(continuation))
+
+                        if click_tracking_params:
+                            continuation_params['context']['clickTracking'] = {
+                                'clickTrackingParams': click_tracking_params}
 
                         yt_info = self._session_post(
                             continuation_url, json=continuation_params).json()
 
+                    debug_info = {
+                        'click_tracking': multi_get(continuation_params, 'context', 'clickTracking'),
+                        'continuation': multi_get(continuation_params, 'continuation')
+                    }
+                    log('debug', 'Continuation: {}'.format(debug_info))
+
                     info = multi_get(
                         yt_info, 'continuationContents', 'liveChatContinuation')
 
+                    logged_in_info = multi_get(
+                        yt_info, 'responseContext', 'serviceTrackingParams', 1, 'params', 0)
+                    log('debug', 'Logged-in info: {}'.format(logged_in_info))
+
                     if not info:
                         log('debug', 'No continuation information found: {}'.format(
                             yt_info))
@@ -1262,8 +1332,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
                 except (JSONDecodeError, RequestException) as e:
                     self.retry(attempt_number, max_attempts, e, retry_timeout)
-                    self.clear_cookies()
-
                     continue
 
             actions = info.get('actions') or []
@@ -1485,12 +1553,16 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 continuation_key = try_get_first_key(cont)
                 continuation_info = cont[continuation_key]
 
+                log('debug', 'Continuation info: {}'.format(continuation_info))
+
                 if continuation_key in self._KNOWN_CHAT_CONTINUATIONS:
 
                     # set new chat continuation
                     # overwrite if there is continuation data
                     continuation = continuation_info.get('continuation')
 
+                    click_tracking_params = continuation_info.get(
+                        'clickTrackingParams') or continuation_info.get('trackingParams')
                     # there is a chat continuation
                     no_continuation = False
 
